//
//  BFEventTrackAdaptor.swift
//  BFAnalyzeKit
//
//  Created by SanW on 2021/12/10.
//

import BFCommonKit
import BFNetRequestKit
import UIKit

public class BFEventTrackAdaptor: NSObject {
    /// 基础埋点上报
    /// - Parameters:
    ///   - logType: 数据库类型
    ///   - businessType: businessType
    ///   - objectType: objectType
    ///   - eventData: eventData
    ///   - pageSource: 页面场景
    ///   - extParams: extParams 扩展字段，为json对象
    ///   - remindmsg: remindmsg 打印提示信息
    /// - Returns: <#description#>
    public class func baseReportUpload(logType: statisticsLogType = .st_log_type_simpleevent, businessType: businessType?, objectType: objectType?, pageSource: PAGESOURCE?, params: [String: Any]? = nil, eventData: [String: Any]? = nil, extParams: [String: Any]? = nil, commonParams: [String: Any]?) {
        BFBaseEventTrack.bf_frontendReportUpload(url: PQENVUtil.shared.commonapi + staticsFrontendReportUrl, logType: logType.rawValue, businessType: businessType?.rawValue, objectType: objectType?.rawValue, pageSource: pageSource?.rawValue, params: params, eventData: eventData, extParams: extParams, commonParams: commonParams)
    }

    /// 视频相关上报
    /// - Parameters:
    ///   - reportLogType: 上报库类型
    ///   - videoData: 视频数据
    ///   - pageSource: 页面
    ///   - businessType: <#businessType description#>
    ///   - objectType: <#objectType description#>
    ///   - extParams: <#extParams description#>
    ///   - shareId: <#shareId description#>
    ///   - videoIds: <#videoIds description#>
    ///   - playId: <#playId description#>
    ///   - headVideoId: <#headVideoId description#>
    public class func videoRelationReportUpload(reportLogType: reportLogType, videoData: BFVideoItemProtocol?, pageSource: PAGESOURCE? = nil, businessType: businessType?, objectType: objectType? = nil, extParams: [String: Any]? = nil, shareId: String? = nil, videoIds: String? = nil, playId: String? = nil, headVideoId: String? = nil, projectId: String?, parentProjectId: String?, rootProjectId: String?, canProduce: Int?, parentVideoId: String?, commonParams: [String: Any]?) {
        DispatchQueue.global().async {
            var tempExtParams: [String: Any] = extParams ?? [:]
            if videoData?.topicData != nil {
                if videoData?.pageSource == .sp_cmunit_joinTopic || pageSource == .sp_cmunit_joinTopic, "\(tempExtParams["topicId"] ?? "")".count <= 0 {
                    tempExtParams["topicId"] = "\(videoData?.topicData?["id"] ?? "")"
                } else if videoData?.pageSource == .sp_cmunit_follow || pageSource == .sp_cmunit_follow, "\(tempExtParams["followedUid"] ?? "")".count <= 0 {
                    tempExtParams["followedUid"] = "\(videoData?.user?["uid"] ?? "")"
                }
            }
            if projectId != nil && (projectId?.count ?? 0) > 0 {
                tempExtParams["projectId"] = projectId ?? ""
            }
            if parentProjectId != nil && (parentProjectId?.count ?? 0) > 0 {
                tempExtParams["parentProjectId"] = parentProjectId ?? ""
            }
            if rootProjectId != nil && (rootProjectId?.count ?? 0) > 0 {
                tempExtParams["rootProjectId"] = rootProjectId ?? ""
            }
            if canProduce != nil {
                tempExtParams["canProduce"] = canProduce ?? 0
            }
            if parentVideoId != nil && (parentVideoId?.count ?? 0) > 0 {
                tempExtParams["parentVideoId"] = parentVideoId ?? ""
            }
            if !tempExtParams.keys.contains("clickedVideoId") {
                tempExtParams["videoId"] = "\(videoData?.id ?? 0)"
            }
            if headVideoId != nil, (headVideoId?.count ?? 0) > 0 {
                tempExtParams["headVideoId"] = (headVideoId ?? "0")
            }
            if objectType == .ot_reproduce_clickButton || objectType == .ot_reproduce_collectionBar || objectType == .ot_reproduce_collectionClicButton || objectType == .ot_reproduce_sameSourceButton {
                baseReportUpload(businessType: businessType, objectType: objectType, pageSource: pageSource != nil ? pageSource! : (videoData?.pageSource ?? .sp_category), extParams: tempExtParams, commonParams: commonParams)
            } else {
                var url: String = PQENVUtil.shared.longvideoapi
                var params: [String: Any] = [:]
                switch reportLogType {
                case .reportLogType_view:
                    url = url + videoViewReportUrl
                    if videoIds != nil, !videoIds!.isEmpty {
                        params["videoIds"] = videoIds
                    } else {
                        params["videoIds"] = "\(videoData?.id ?? 0)"
                    }
                    params["viewId"] = ("\(Date().timeIntervalSince1970)" + "\(videoData?.id ?? 0)viewId" + "\(arc4random_uniform(1_000_000_000))")
                case .reportLogType_realPlay:
                    url = url + videoRealPlayReportUrl
                    if businessType != nil {
                        params["actionTriggerType"] = businessType?.rawValue
                    }
                case .reportLogType_play:
                    url = url + videoPlayReportUrl
                case .reportLogType_Action:
                    url = url + videoActionReportUrl
                    if businessType != nil, businessType == .bt_videoShareH5 || businessType == .bt_videoShareFriend {
                        params["shareId"] = shareId
                        params["rootLaunchShareId"] = shareId
                        params["parentShareId"] = shareId
                        params["shareDepth"] = "0"
                        params["videoId"] = videoIds
                    }
                    if businessType != nil {
                        params["businessType"] = businessType?.rawValue
                    }
                case .reportLogType_Frontend:
                    url = PQENVUtil.shared.commonapi + staticsFrontendReportUrl
                    params["LogType"] = 30
                    params["eventData"] = bf_dictionaryToJsonString(["businessType": businessType?.rawValue ?? ""])
                }
                BFBaseEventTrack.bf_videoReportUpload(url: url, videoId: "\(videoData?.id ?? 0)", headVideoId: headVideoId ?? videoData?.headVideoId, playId: playId ?? "", recommendId: videoData?.recommendId, recommendLogVO: videoData?.recommendLogVO, flowPool: videoData?.flowPool, abInfoData: videoData?.abInfoData, measureType: videoData?.measureType, measureId: videoData?.measureId, businessType: businessType?.rawValue, objectType: objectType?.rawValue, pageSource: (videoData?.pageSource ?? .sp_category).rawValue, extParams: tempExtParams, commonParams: commonParams)
            }
        }
    }

    // 分享上报
    // - Parameters:
    //   - isShareVideo: 是否是分享视频
    //   - screenType: 分享场景 1-分享视频/用户 2-分享视频到朋友圈 3-分享视频到好友
    //   - videoId: 视频Id
    //   - pageSource: 页面枚举
    //   - recommendId: <#recommendId description#>
    //   - recommendLogVO: <#recommendLogVO description#>
    //   - abInfoData: <#abInfoData description#>
    //   - measureType: <#measureType description#>
    //   - measureId: <#measureId description#>
    //   - businessType: <#businessType description#>
    //   - targetUid: <#targetUid description#>
    //   - shareId: <#shareId description#>
    public class func shareReportUpload(isShareVideo: Bool = true, screenType: Int = 1, videoId: String, pageSource: PAGESOURCE, recommendId: String?, recommendLogVO: String?, flowPool: String?, abInfoData: String?, measureType: Int?, measureId: Int?, businessType: businessType?, targetUid: Int?, shareId: String = "", playId: String?, wxOpenId: String?, extParams: [String: Any]? = nil, commonParams: [String: Any]?) {
        DispatchQueue.global().async {
            var url: String = PQENVUtil.shared.longvideoapi
            switch screenType {
            case 1:
                url = url + userShareReportUrl
            case 2:
                url = url + userShareH5ReportUrl
            case 3:
                url = url + userShareFriendReportUrl
            default:
                break
            }
            BFBaseEventTrack.bf_shareReportUpload(url: url, isShareVideo: isShareVideo, screenType: screenType, videoId: videoId, recommendId: recommendId, recommendLogVO: recommendLogVO, flowPool: flowPool, abInfoData: abInfoData, measureType: measureType, measureId: measureId, targetUid: targetUid, shareId: shareId, playId: playId, wxOpenId: wxOpenId, businessType: businessType?.rawValue, pageSource: pageSource.rawValue, extParams: extParams, commonParams: commonParams)
        }
    }

    /// DNS上报
    /// - Returns: <#description#>
    public class func dnsReportUpload(commonParams: [String: Any]?) {
        DispatchQueue.global().async {
            let speedExtParams = bf_parseDNS(hostUrl: "speed.piaoquantv.com")
            let rescdnExtParams = bf_parseDNS(hostUrl: "rescdn.yishihui.com")
            if speedExtParams != nil {
                baseReportUpload(businessType: .bt_dnsParseCostTime, objectType: nil, pageSource: nil, eventData: nil, extParams: speedExtParams, commonParams: commonParams)
            }
            if rescdnExtParams != nil {
                baseReportUpload(businessType: .bt_dnsParseCostTime, objectType: nil, pageSource: nil, eventData: nil, extParams: rescdnExtParams, commonParams: commonParams)
            }
        }
    }

    
    /// app冷热启动上报
    /// - Parameters:
    ///   - isFirstLaunch: <#isFirstLaunch description#>
    ///   - firstParams: <#firstParams description#>
    ///   - isHotLaunch: <#isHotLaunch description#>
    ///   - launchType: <#launchType description#>
    ///   - commonParams: <#commonParams description#>
    class public func appLaunchReportUpload(isFirstLaunch: Int = 1, firstParams: [String: Any]? = nil, isHotLaunch: Bool = false, launchType: String = "userActiveOpen", commonParams: [String: Any]?) {
        BFBaseEventTrack.appLaunchReportUpload(url: PQENVUtil.shared.commonapi + staticsFrontendReportUrl, isFirstLaunch: isFirstLaunch, firstParams: firstParams, isHotLaunch: isHotLaunch, launchType: launchType, commonParams: commonParams)
    }

    /// 推送点击数据上报
    /// - Parameters:
    ///   - pushId: 推送Id
    ///   - pushTargetType: 1-推送单个视频 2-整体关注有更新 3-关注单个up主有更新(暂废弃) 4-订阅某人有更新
    ///   - pushBrand: 推送平台 APPLE_TYPE
    ///   - pushTargetId: pushTargetType == 1 视频ID,pushTargetType == 4 用户ID
    ///   - bizParam 扩展参数
    /// - Returns: <#description#>
    public class func reportPushActionUpload(pushId: String, pushTargetType: Int, pushBrand: String, pushTargetId: String?, bizParam: [String: Any]? = nil, commonParams: [String: Any]?) {
        BFBaseEventTrack.bf_pushActionReportUpload(url: PQENVUtil.shared.longvideoapi + pushActionReportUrl, pushId: pushId, pushTargetType: pushTargetType, pushBrand: pushBrand, pushTargetId: pushTargetId, bizParam: bizParam, commonParams: commonParams)
    }

    /// 上报deviceToken
    /// - Parameter registerId: 设备id
    /// - Parameter deviceToken: <#deviceToken description#>
    /// - Returns: <#description#>
    public class func reportPushDeviceTokenUpload(registerId: String, deviceToken: String, brand: String, commonParams: [String: Any]?, completeHander: @escaping (_ isSuccess: Bool) -> Void) {
        BFBaseEventTrack.bf_deviceTokenReportUpload(url: PQENVUtil.shared.longvideoapi + pushDeviceTokenReportUrl, registerId: registerId, deviceToken: deviceToken, brand: brand, commonParams: commonParams, completeHander: completeHander)
    }

    /// 搜索上报
    /// - Parameters:
    ///   - keyWord: 搜索词
    ///   - searchType: 1 热搜词搜索 2 历史记录 3 普通搜索
    ///   - searchNumber: 数量
    ///   - reportType: 1 into 2 click 3 show
    /// - Returns: <#description#>
    public class func searchReportUpload(keyWord: String, searchType: Int, searchNumber: Int = 10, reportType: Int = 2, commonParams: [String: Any]?) {
        BFBaseEventTrack.bf_searchReportUpload(url: PQENVUtil.shared.longvideoapi + searchReportUrl, keyWord: keyWord, searchType: searchType, searchNumber: searchNumber, reportType: reportType, commonParams: commonParams)
    }

    /// 发布视频的上报
    /// - Parameters:
    ///   - projectId:项目ID-发布创作的视频时必传，会在进入创作工具页时生成，以app_no_projectdata为前缀
    ///   - businessType: <#businessType description#>
    ///   - ossInfo: <#ossInfo description#>
    ///   - params: <#params description#>
    /// - Returns: <#description#>
    public class func publishReportUpload(projectId: String?, businessType: businessType, ossInfo: [String: Any], params: [String: Any], commonParams: [String: Any]?) {
        var extParams: [String: Any] = ["ossInfo": bf_dictionaryToJsonString(ossInfo) ?? "", "params": bf_dictionaryToJsonString(params) ?? ""]
        if projectId != nil {
            extParams["projectId"] = projectId
        }
        if !extParams.keys.contains("source") {
            extParams["source"] = projectId != nil ? "videoCompose" : "videoUpload"
        }
        baseReportUpload(businessType: businessType, objectType: nil, pageSource: nil, extParams: extParams, commonParams: commonParams)
    }

    /// 站内消息埋点上报
    /// - Parameters:
    ///   - messageIds: 消息Id，多个用逗号分隔
    ///   - clickId: 子入口点击ID，标识一次子入口点击动作。子入口内消息列表中的消息点击行为都带有此字段，分享空间消息除外
    ///   - messageType: 消息类型
    ///   - messageSubType: 消息子类型
    ///   - actionType: 动作类型（backendCreate：后端构建；backendReturn：后端返回；frontendPull：前端拉取；view：曝光；click：点击）
    ///   - objectType: <#objectType description#>
    ///   - pageSource: <#pageSource description#>
    ///   - readStatus: 已读状态：1：页面上显示未读 2：页面上显示已读
    ///   - eventData: 扩展数据，json格式，日志系统里会展开存储
    ///   - extParams: 扩展字段 json格式
    ///   - remindmsg: 打印提示信息
    /// - Returns: <#description#>
    public class func messageReportUpload(messageIds: String?, clickId: String?, messageType: messageType?, messageSubType: messageSubType?, actionType: actionType?, objectType: objectType?, pageSource: PAGESOURCE?, readStatus: Int = 1, eventData: [String: Any]? = nil, extParams: [String: Any]? = nil, commonParams: [String: Any]?) {
        BFBaseEventTrack.bf_messageReportUpload(url: PQENVUtil.shared.longvideoapi + messagePeportUrl, messageIds: messageIds, clickId: clickId, messageType: messageType?.rawValue, messageSubType: messageSubType?.rawValue, actionType: actionType?.rawValue, readStatus: readStatus, objectType: objectType?.rawValue, pageSource: pageSource?.rawValue, eventData: eventData, extParams: extParams, commonParams: commonParams)
    }
}
